"""VTAP100 Configuration Models.

This package contains Pydantic models for all VTAP100 configuration options:
- vas: Apple VAS (Value Added Services) configuration
- smarttap: Google Smart Tap configuration
- keyboard: Keyboard emulation settings
- nfc: NFC tag settings
- desfire: MIFARE DESFire settings
- feedback: LED and buzzer settings
- config: Main configuration model combining all settings
"""

# Models will be exported here as they are implemented
__all__: list[str] = []
